@@ -1,7 +1,9 @@
 pub use filebuf::*;
 pub use read::default_settings;
 pub use read::read_ods;
+pub use read::read_ods_buf;
 pub use write::write_ods;
+pub use write::write_ods_buf;
 
 pub use crate::error::OdsError;
 
