@@ -1,6 +1,6 @@
 use std::collections::HashMap;
 use std::io;
-use std::io::Write;
+use std::io::{Cursor, Seek, Write};
 use std::path::Path;
 
 use chrono::NaiveDateTime;
@@ -20,22 +20,39 @@ use crate::style::{
 use crate::validation::ValidationDisplay;
 use crate::xmltree::{XmlContent, XmlTag};
 use crate::{ucell, Length, SCell, Sheet, Value, ValueFormat, ValueType, Visibility, WorkBook};
+use std::fs::File;
 
-type OdsWriter = ZipOut;
-type XmlOdsWriter<'a> = XmlWriter<ZipWrite<'a>>;
+type OdsWriter<W> = ZipOut<W>;
+type XmlOdsWriter<'a, W> = XmlWriter<ZipWrite<'a, W>>;
+
+/// Writes the ODS file into a supplied buffer.
+pub fn write_ods_buf(book: &mut WorkBook, buf: Vec<u8>) -> Result<Vec<u8>, OdsError> {
+    let zip_writer = ZipOut::<Cursor<Vec<u8>>>::new_buf(buf)?;
+    Ok(write_ods_impl(book, zip_writer)?.into_inner())
+}
 
 /// Writes the ODS file.
 ///
 /// All the parts are written to a temp directory and then zipped together.
 ///
 pub fn write_ods<P: AsRef<Path>>(book: &mut WorkBook, ods_path: P) -> Result<(), OdsError> {
+    let zip_writer = ZipOut::<File>::new_file(ods_path.as_ref())?;
+    write_ods_impl(book, zip_writer)?;
+    Ok(())
+}
+
+/// Writes the ODS file.
+///
+/// All the parts are written to a temp directory and then zipped together.
+///
+fn write_ods_impl<W: Write + Seek>(
+    book: &mut WorkBook,
+    mut zip_writer: OdsWriter<W>,
+) -> Result<W, OdsError> {
     sanity_checks(book)?;
 
     store_derived(book)?;
 
-    let mut zip_writer = OdsWriter::new(ods_path.as_ref())?;
-    // let mut zip_writer = TempZip::new(ods_path.as_ref())?;
-
     // copy all buffered data from the original.
     copy_workbook(&book, &mut zip_writer)?;
     // write the rest, if necessary.
@@ -49,9 +66,7 @@ pub fn write_ods<P: AsRef<Path>>(book: &mut WorkBook, ods_path: P) -> Result<(),
     write_ods_styles(&book, &mut zip_writer)?;
     write_ods_content(&book, &mut zip_writer)?;
 
-    zip_writer.zip()?;
-
-    Ok(())
+    Ok(zip_writer.zip()?)
 }
 
 fn sanity_checks(book: &mut WorkBook) -> Result<(), OdsError> {
@@ -164,7 +179,10 @@ fn store_derived(book: &mut WorkBook) -> Result<(), OdsError> {
     Ok(())
 }
 
-fn copy_workbook(book: &WorkBook, zip_writer: &mut OdsWriter) -> Result<(), OdsError> {
+fn copy_workbook<W: Write + Seek>(
+    book: &WorkBook,
+    zip_writer: &mut OdsWriter<W>,
+) -> Result<(), OdsError> {
     for filebuf in book.filebuf.iter() {
         match filebuf {
             FileBufEntry::Dir(name) => {
@@ -180,7 +198,10 @@ fn copy_workbook(book: &WorkBook, zip_writer: &mut OdsWriter) -> Result<(), OdsE
     Ok(())
 }
 
-fn write_mimetype(book: &WorkBook, zip_out: &mut OdsWriter) -> Result<(), io::Error> {
+fn write_mimetype<W: Write + Seek>(
+    book: &WorkBook,
+    zip_out: &mut OdsWriter<W>,
+) -> Result<(), io::Error> {
     if !book.filebuf.contains("mimetype") {
         let mut w = zip_out.start_file(
             "mimetype",
@@ -194,7 +215,10 @@ fn write_mimetype(book: &WorkBook, zip_out: &mut OdsWriter) -> Result<(), io::Er
     Ok(())
 }
 
-fn write_manifest(book: &WorkBook, zip_out: &mut OdsWriter) -> Result<(), OdsError> {
+fn write_manifest<W: Write + Seek>(
+    book: &WorkBook,
+    zip_out: &mut OdsWriter<W>,
+) -> Result<(), OdsError> {
     if !book.filebuf.contains("META-INF/manifest.xml") {
         zip_out.add_directory("META-INF", FileOptions::default())?;
         let w = zip_out.start_file("META-INF/manifest.xml", FileOptions::default())?;
@@ -251,7 +275,10 @@ fn write_manifest(book: &WorkBook, zip_out: &mut OdsWriter) -> Result<(), OdsErr
     Ok(())
 }
 
-fn write_manifest_rdf(book: &WorkBook, zip_out: &mut OdsWriter) -> Result<(), OdsError> {
+fn write_manifest_rdf<W: Write + Seek>(
+    book: &WorkBook,
+    zip_out: &mut OdsWriter<W>,
+) -> Result<(), OdsError> {
     if !book.filebuf.contains("manifest.rdf") {
         let w = zip_out.start_file("manifest.rdf", FileOptions::default())?;
 
@@ -304,7 +331,10 @@ fn write_manifest_rdf(book: &WorkBook, zip_out: &mut OdsWriter) -> Result<(), Od
     Ok(())
 }
 
-fn write_meta(book: &WorkBook, zip_out: &mut OdsWriter) -> Result<(), OdsError> {
+fn write_meta<W: Write + Seek>(
+    book: &WorkBook,
+    zip_out: &mut OdsWriter<W>,
+) -> Result<(), OdsError> {
     if !book.filebuf.contains("meta.xml") {
         let w = zip_out.start_file("meta.xml", FileOptions::default())?;
 
@@ -348,7 +378,10 @@ fn write_meta(book: &WorkBook, zip_out: &mut OdsWriter) -> Result<(), OdsError>
     Ok(())
 }
 
-fn write_settings(book: &WorkBook, zip_out: &mut OdsWriter) -> Result<(), OdsError> {
+fn write_settings<W: Write + Seek>(
+    book: &WorkBook,
+    zip_out: &mut OdsWriter<W>,
+) -> Result<(), OdsError> {
     let w = zip_out.start_file("settings.xml", FileOptions::default())?;
 
     let mut xml_out = XmlWriter::new(w);
@@ -394,10 +427,10 @@ fn write_settings(book: &WorkBook, zip_out: &mut OdsWriter) -> Result<(), OdsErr
     Ok(())
 }
 
-fn write_config_item_set(
+fn write_config_item_set<W: Write + Seek>(
     name: &str,
     set: &ConfigItem,
-    xml_out: &mut XmlOdsWriter,
+    xml_out: &mut XmlOdsWriter<W>,
 ) -> Result<(), OdsError> {
     xml_out.elem("config:config-item-set")?;
     xml_out.attr("config:name", name)?;
@@ -419,10 +452,10 @@ fn write_config_item_set(
     Ok(())
 }
 
-fn write_config_item_map_indexed(
+fn write_config_item_map_indexed<W: Write + Seek>(
     name: &str,
     vec: &ConfigItem,
-    xml_out: &mut XmlOdsWriter,
+    xml_out: &mut XmlOdsWriter<W>,
 ) -> Result<(), OdsError> {
     xml_out.elem("config:config-item-map-indexed")?;
     xml_out.attr("config:name", name)?;
@@ -456,10 +489,10 @@ fn write_config_item_map_indexed(
     Ok(())
 }
 
-fn write_config_item_map_named(
+fn write_config_item_map_named<W: Write + Seek>(
     name: &str,
     map: &ConfigItem,
-    xml_out: &mut XmlOdsWriter,
+    xml_out: &mut XmlOdsWriter<W>,
 ) -> Result<(), OdsError> {
     xml_out.elem("config:config-item-map-named")?;
     xml_out.attr("config:name", name)?;
@@ -485,10 +518,10 @@ fn write_config_item_map_named(
     Ok(())
 }
 
-fn write_config_item_map_entry(
+fn write_config_item_map_entry<W: Write + Seek>(
     name: Option<&String>,
     map_entry: &ConfigItem,
-    xml_out: &mut XmlOdsWriter,
+    xml_out: &mut XmlOdsWriter<W>,
 ) -> Result<(), OdsError> {
     xml_out.elem("config:config-item-map-entry")?;
     if let Some(name) = name {
@@ -512,10 +545,10 @@ fn write_config_item_map_entry(
     Ok(())
 }
 
-fn write_config_item(
+fn write_config_item<W: Write + Seek>(
     name: &str,
     value: &ConfigValue,
-    xml_out: &mut XmlOdsWriter,
+    xml_out: &mut XmlOdsWriter<W>,
 ) -> Result<(), OdsError> {
     let is_empty = match value {
         ConfigValue::Base64Binary(t) => t.is_empty(),
@@ -603,7 +636,10 @@ fn write_config_item(
 //    Ok(())
 //}
 
-fn write_ods_styles(book: &WorkBook, zip_out: &mut OdsWriter) -> Result<(), OdsError> {
+fn write_ods_styles<W: Write + Seek>(
+    book: &WorkBook,
+    zip_out: &mut OdsWriter<W>,
+) -> Result<(), OdsError> {
     let w = zip_out.start_file("styles.xml", FileOptions::default())?;
 
     let mut xml_out = XmlWriter::new(w);
@@ -623,6 +659,9 @@ fn write_ods_styles(book: &WorkBook, zip_out: &mut OdsWriter) -> Result<(), OdsE
         "xmlns:fo",
         "urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0",
     )?;
+    xml_out.attr("xmlns:ooo", "http://openoffice.org/2004/office")?;
+    xml_out.attr("xmlns:xlink", "http://www.w3.org/1999/xlink")?;
+    xml_out.attr("xmlns:dc", "http://purl.org/dc/elements/1.1/")?;
     xml_out.attr(
         "xmlns:style",
         "urn:oasis:names:tc:opendocument:xmlns:style:1.0",
@@ -643,6 +682,7 @@ fn write_ods_styles(book: &WorkBook, zip_out: &mut OdsWriter) -> Result<(), OdsE
         "xmlns:chart",
         "urn:oasis:names:tc:opendocument:xmlns:chart:1.0",
     )?;
+    xml_out.attr("xmlns:rpt", "http://openoffice.org/2005/report")?;
     xml_out.attr(
         "xmlns:table",
         "urn:oasis:names:tc:opendocument:xmlns:table:1.0",
@@ -651,11 +691,15 @@ fn write_ods_styles(book: &WorkBook, zip_out: &mut OdsWriter) -> Result<(), OdsE
         "xmlns:number",
         "urn:oasis:names:tc:opendocument:xmlns:datastyle:1.0",
     )?;
+    xml_out.attr("xmlns:ooow", "http://openoffice.org/2004/writer")?;
+    xml_out.attr("xmlns:oooc", "http://openoffice.org/2004/calc")?;
     xml_out.attr("xmlns:of", "urn:oasis:names:tc:opendocument:xmlns:of:1.2")?;
+    xml_out.attr("xmlns:tableooo", "http://openoffice.org/2009/table")?;
     xml_out.attr(
         "xmlns:calcext",
         "urn:org:documentfoundation:names:experimental:calc:xmlns:calcext:1.0",
     )?;
+    xml_out.attr("xmlns:drawooo", "http://openoffice.org/2010/draw")?;
     xml_out.attr(
         "xmlns:loext",
         "urn:org:documentfoundation:names:experimental:office:xmlns:loext:1.0",
@@ -664,6 +708,7 @@ fn write_ods_styles(book: &WorkBook, zip_out: &mut OdsWriter) -> Result<(), OdsE
         "xmlns:field",
         "urn:openoffice:names:experimental:ooo-ms-interop:xmlns:field:1.0",
     )?;
+    xml_out.attr("xmlns:math", "http://www.w3.org/1998/Math/MathML")?;
     xml_out.attr(
         "xmlns:form",
         "urn:oasis:names:tc:opendocument:xmlns:form:1.0",
@@ -672,6 +717,10 @@ fn write_ods_styles(book: &WorkBook, zip_out: &mut OdsWriter) -> Result<(), OdsE
         "xmlns:script",
         "urn:oasis:names:tc:opendocument:xmlns:script:1.0",
     )?;
+    xml_out.attr("xmlns:dom", "http://www.w3.org/2001/xml-events")?;
+    xml_out.attr("xmlns:xhtml", "http://www.w3.org/1999/xhtml")?;
+    xml_out.attr("xmlns:grddl", "http://www.w3.org/2003/g/data-view#")?;
+    xml_out.attr("xmlns:css3t", "http://www.w3.org/TR/css3-text/")?;
     xml_out.attr(
         "xmlns:presentation",
         "urn:oasis:names:tc:opendocument:xmlns:presentation:1.0",
@@ -721,7 +770,10 @@ fn write_ods_styles(book: &WorkBook, zip_out: &mut OdsWriter) -> Result<(), OdsE
     Ok(())
 }
 
-fn write_ods_content(book: &WorkBook, zip_out: &mut OdsWriter) -> Result<(), OdsError> {
+fn write_ods_content<W: Write + Seek>(
+    book: &WorkBook,
+    zip_out: &mut OdsWriter<W>,
+) -> Result<(), OdsError> {
     let w = zip_out.start_file("content.xml", FileOptions::default())?;
     let mut xml_out = XmlWriter::new(w);
 
@@ -740,6 +792,9 @@ fn write_ods_content(book: &WorkBook, zip_out: &mut OdsWriter) -> Result<(), Ods
         "xmlns:fo",
         "urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0",
     )?;
+    xml_out.attr("xmlns:ooo", "http://openoffice.org/2004/office")?;
+    xml_out.attr("xmlns:xlink", "http://www.w3.org/1999/xlink")?;
+    xml_out.attr("xmlns:dc", "http://purl.org/dc/elements/1.1/")?;
     xml_out.attr(
         "xmlns:style",
         "urn:oasis:names:tc:opendocument:xmlns:style:1.0",
@@ -748,6 +803,10 @@ fn write_ods_content(book: &WorkBook, zip_out: &mut OdsWriter) -> Result<(), Ods
         "xmlns:text",
         "urn:oasis:names:tc:opendocument:xmlns:text:1.0",
     )?;
+    xml_out.attr(
+        "xmlns:draw",
+        "urn:oasis:names:tc:opendocument:xmlns:drawing:1.0",
+    )?;
     xml_out.attr(
         "xmlns:dr3d",
         "urn:oasis:names:tc:opendocument:xmlns:dr3d:1.0",
@@ -760,15 +819,24 @@ fn write_ods_content(book: &WorkBook, zip_out: &mut OdsWriter) -> Result<(), Ods
         "xmlns:chart",
         "urn:oasis:names:tc:opendocument:xmlns:chart:1.0",
     )?;
+    xml_out.attr("xmlns:rpt", "http://openoffice.org/2005/report")?;
     xml_out.attr(
         "xmlns:table",
         "urn:oasis:names:tc:opendocument:xmlns:table:1.0",
     )?;
+    xml_out.attr(
+        "xmlns:number",
+        "urn:oasis:names:tc:opendocument:xmlns:datastyle:1.0",
+    )?;
+    xml_out.attr("xmlns:ooow", "http://openoffice.org/2004/writer")?;
+    xml_out.attr("xmlns:oooc", "http://openoffice.org/2004/calc")?;
     xml_out.attr("xmlns:of", "urn:oasis:names:tc:opendocument:xmlns:of:1.2")?;
+    xml_out.attr("xmlns:tableooo", "http://openoffice.org/2009/table")?;
     xml_out.attr(
         "xmlns:calcext",
         "urn:org:documentfoundation:names:experimental:calc:xmlns:calcext:1.0",
     )?;
+    xml_out.attr("xmlns:drawooo", "http://openoffice.org/2010/draw")?;
     xml_out.attr(
         "xmlns:loext",
         "urn:org:documentfoundation:names:experimental:office:xmlns:loext:1.0",
@@ -777,6 +845,7 @@ fn write_ods_content(book: &WorkBook, zip_out: &mut OdsWriter) -> Result<(), Ods
         "xmlns:field",
         "urn:openoffice:names:experimental:ooo-ms-interop:xmlns:field:1.0",
     )?;
+    xml_out.attr("xmlns:math", "http://www.w3.org/1998/Math/MathML")?;
     xml_out.attr(
         "xmlns:form",
         "urn:oasis:names:tc:opendocument:xmlns:form:1.0",
@@ -785,14 +854,21 @@ fn write_ods_content(book: &WorkBook, zip_out: &mut OdsWriter) -> Result<(), Ods
         "xmlns:script",
         "urn:oasis:names:tc:opendocument:xmlns:script:1.0",
     )?;
+    xml_out.attr("xmlns:dom", "http://www.w3.org/2001/xml-events")?;
+    xml_out.attr("xmlns:xforms", "http://www.w3.org/2002/xforms")?;
+    xml_out.attr("xmlns:xsd", "http://www.w3.org/2001/XMLSchema")?;
+    xml_out.attr("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance")?;
+    xml_out.attr(
+        "xmlns:formx",
+        "urn:openoffice:names:experimental:ooxml-odf-interop:xmlns:form:1.0",
+    )?;
+    xml_out.attr("xmlns:xhtml", "http://www.w3.org/1999/xhtml")?;
+    xml_out.attr("xmlns:grddl", "http://www.w3.org/2003/g/data-view#")?;
+    xml_out.attr("xmlns:css3t", "http://www.w3.org/TR/css3-text/")?;
     xml_out.attr(
         "xmlns:presentation",
         "urn:oasis:names:tc:opendocument:xmlns:presentation:1.0",
     )?;
-    xml_out.attr(
-        "xmlns:number",
-        "urn:oasis:names:tc:opendocument:xmlns:datastyle:1.0",
-    )?;
 
     xml_out.attr("office:version", book.version())?;
 
@@ -863,7 +939,10 @@ fn write_ods_content(book: &WorkBook, zip_out: &mut OdsWriter) -> Result<(), Ods
     Ok(())
 }
 
-fn write_content_validations(book: &WorkBook, xml_out: &mut XmlOdsWriter) -> Result<(), OdsError> {
+fn write_content_validations<W: Write + Seek>(
+    book: &WorkBook,
+    xml_out: &mut XmlOdsWriter<W>,
+) -> Result<(), OdsError> {
     xml_out.elem("table:content-validations")?;
 
     for valid in book.validations.values() {
@@ -946,7 +1025,11 @@ pub(crate) fn remove_outlooped(ranges: &mut Vec<CellRange>, row: ucell, col: uce
         .collect();
 }
 
-fn write_sheet(book: &WorkBook, sheet: &Sheet, xml_out: &mut XmlOdsWriter) -> Result<(), OdsError> {
+fn write_sheet<W: Write + Seek>(
+    book: &WorkBook,
+    sheet: &Sheet,
+    xml_out: &mut XmlOdsWriter<W>,
+) -> Result<(), OdsError> {
     xml_out.elem("table:table")?;
     xml_out.attr_esc("table:name", &*sheet.name)?;
     if let Some(style) = &sheet.style {
@@ -985,6 +1068,7 @@ fn write_sheet(book: &WorkBook, sheet: &Sheet, xml_out: &mut XmlOdsWriter) -> Re
     // table-row + table-cell
     let mut first_cell = true;
     let mut last_r: ucell = 0;
+    let mut last_r_repeat: ucell = 1;
     let mut last_c: ucell = 0;
 
     for ((cur_row, cur_col), cell) in sheet.data.iter() {
@@ -1031,7 +1115,18 @@ fn write_sheet(book: &WorkBook, sheet: &Sheet, xml_out: &mut XmlOdsWriter) -> Re
 
         // Any empty rows before this one?
         if backward_dr > 0 {
-            write_empty_rows_before(sheet, *cur_row, first_cell, backward_dr, max_cell, xml_out)?;
+            // If the last row had a repeat counter the distance is reduced.
+            // We should not add any extra empty rows.
+            if last_r_repeat - 1 < backward_dr {
+                write_empty_rows_before(
+                    sheet,
+                    *cur_row,
+                    first_cell,
+                    backward_dr - last_r_repeat + 1,
+                    max_cell,
+                    xml_out,
+                )?;
+            }
         }
 
         // Start a new row if there is a delta or we are at the start.
@@ -1068,6 +1163,11 @@ fn write_sheet(book: &WorkBook, sheet: &Sheet, xml_out: &mut XmlOdsWriter) -> Re
 
         first_cell = false;
         last_r = *cur_row;
+        last_r_repeat = if let Some(row_header) = sheet.row_header.get(&cur_row) {
+            row_header.repeat
+        } else {
+            1
+        };
         last_c = *cur_col;
     }
 
@@ -1082,10 +1182,10 @@ fn write_sheet(book: &WorkBook, sheet: &Sheet, xml_out: &mut XmlOdsWriter) -> Re
     Ok(())
 }
 
-fn write_empty_cells(
+fn write_empty_cells<W: Write + Seek>(
     mut forward_dc: u32,
     hidden_cols: u32,
-    xml_out: &mut XmlOdsWriter,
+    xml_out: &mut XmlOdsWriter<W>,
 ) -> Result<(), OdsError> {
     // split between hidden and regular cells.
     if hidden_cols >= forward_dc {
@@ -1111,11 +1211,11 @@ fn write_empty_cells(
     Ok(())
 }
 
-fn write_start_current_row(
+fn write_start_current_row<W: Write + Seek>(
     sheet: &Sheet,
     cur_row: ucell,
     backward_dc: u32,
-    xml_out: &mut XmlOdsWriter,
+    xml_out: &mut XmlOdsWriter<W>,
 ) -> Result<(), OdsError> {
     // Start of headers
     if let Some(header_rows) = &sheet.header_rows {
@@ -1153,11 +1253,11 @@ fn write_start_current_row(
     Ok(())
 }
 
-fn write_end_last_row(
+fn write_end_last_row<W: Write + Seek>(
     sheet: &Sheet,
     cur_row: u32,
     backward_dr: u32,
-    xml_out: &mut XmlOdsWriter,
+    xml_out: &mut XmlOdsWriter<W>,
 ) -> Result<(), OdsError> {
     xml_out.end_elem("table:table-row")?;
 
@@ -1172,10 +1272,10 @@ fn write_end_last_row(
     Ok(())
 }
 
-fn write_end_current_row(
+fn write_end_current_row<W: Write + Seek>(
     sheet: &Sheet,
     cur_row: u32,
-    xml_out: &mut XmlOdsWriter,
+    xml_out: &mut XmlOdsWriter<W>,
 ) -> Result<(), OdsError> {
     xml_out.end_elem("table:table-row")?;
 
@@ -1189,13 +1289,13 @@ fn write_end_current_row(
     Ok(())
 }
 
-fn write_empty_rows_before(
+fn write_empty_rows_before<W: Write + Seek>(
     sheet: &Sheet,
     cur_row: ucell,
     first_cell: bool,
     mut backward_dr: u32,
     max_cell: (ucell, ucell),
-    xml_out: &mut XmlOdsWriter,
+    xml_out: &mut XmlOdsWriter<W>,
 ) -> Result<(), OdsError> {
     // Empty rows in between are 1 less than the delta, except at the very start.
     let mut corr = if first_cell { 0u32 } else { 1u32 };
@@ -1251,12 +1351,12 @@ fn write_empty_rows_before(
     Ok(())
 }
 
-fn write_empty_row(
+fn write_empty_row<W: Write + Seek>(
     sheet: &Sheet,
     cur_row: ucell,
     empty_count: u32,
     max_cell: (u32, u32),
-    xml_out: &mut XmlOdsWriter,
+    xml_out: &mut XmlOdsWriter<W>,
 ) -> Result<(), OdsError> {
     xml_out.elem("table:table-row")?;
     xml_out.attr("table:number-rows-repeated", &empty_count.to_string())?;
@@ -1285,7 +1385,10 @@ fn write_empty_row(
     Ok(())
 }
 
-fn write_xmltag(x: &XmlTag, xml_out: &mut XmlOdsWriter) -> Result<(), OdsError> {
+fn write_xmltag<W: Write + Seek>(
+    x: &XmlTag,
+    xml_out: &mut XmlOdsWriter<W>,
+) -> Result<(), OdsError> {
     if x.is_empty() {
         xml_out.empty(x.name())?;
     } else {
@@ -1313,10 +1416,10 @@ fn write_xmltag(x: &XmlTag, xml_out: &mut XmlOdsWriter) -> Result<(), OdsError>
     Ok(())
 }
 
-fn write_table_columns(
+fn write_table_columns<W: Write + Seek>(
     sheet: &Sheet,
     max_cell: (ucell, ucell),
-    xml_out: &mut XmlOdsWriter,
+    xml_out: &mut XmlOdsWriter<W>,
 ) -> Result<(), OdsError> {
     // table:table-column
     for c in 0..max_cell.1 {
@@ -1355,11 +1458,11 @@ fn write_table_columns(
 }
 
 #[allow(clippy::single_char_add_str)]
-fn write_cell(
+fn write_cell<W: Write + Seek>(
     book: &WorkBook,
     cell: &SCell,
     is_hidden: bool,
-    xml_out: &mut XmlOdsWriter,
+    xml_out: &mut XmlOdsWriter<W>,
 ) -> Result<(), OdsError> {
     let tag = if is_hidden {
         "table:covered-table-cell"
@@ -1419,7 +1522,9 @@ fn write_cell(
             }
         }
         Value::TextXml(t) => {
-            write_xmltag(t, xml_out)?;
+            for tt in t.iter() {
+                write_xmltag(tt, xml_out)?;
+            }
         }
         Value::DateTime(d) => {
             xml_out.attr("office:value-type", "date")?;
@@ -1521,10 +1626,10 @@ fn write_cell(
     Ok(())
 }
 
-fn write_font_decl(
+fn write_font_decl<W: Write + Seek>(
     fonts: &HashMap<String, FontFaceDecl>,
     origin: StyleOrigin,
-    xml_out: &mut XmlOdsWriter,
+    xml_out: &mut XmlOdsWriter<W>,
 ) -> Result<(), OdsError> {
     for font in fonts.values().filter(|s| s.origin() == origin) {
         xml_out.empty("style:font-face")?;
@@ -1536,11 +1641,11 @@ fn write_font_decl(
     Ok(())
 }
 
-fn write_styles(
+fn write_styles<W: Write + Seek>(
     book: &WorkBook,
     origin: StyleOrigin,
     styleuse: StyleUse,
-    xml_out: &mut XmlOdsWriter,
+    xml_out: &mut XmlOdsWriter<W>,
 ) -> Result<(), OdsError> {
     for style in book.tablestyles.values() {
         if style.origin() == origin && style.styleuse() == styleuse {
@@ -1590,7 +1695,10 @@ fn write_styles(
     Ok(())
 }
 
-fn write_tablestyle(style: &TableStyle, xml_out: &mut XmlOdsWriter) -> Result<(), OdsError> {
+fn write_tablestyle<W: Write + Seek>(
+    style: &TableStyle,
+    xml_out: &mut XmlOdsWriter<W>,
+) -> Result<(), OdsError> {
     if style.styleuse() == StyleUse::Default {
         xml_out.elem("style:default-style")?;
     } else {
@@ -1623,7 +1731,10 @@ fn write_tablestyle(style: &TableStyle, xml_out: &mut XmlOdsWriter) -> Result<()
     Ok(())
 }
 
-fn write_rowstyle(style: &RowStyle, xml_out: &mut XmlOdsWriter) -> Result<(), OdsError> {
+fn write_rowstyle<W: Write + Seek>(
+    style: &RowStyle,
+    xml_out: &mut XmlOdsWriter<W>,
+) -> Result<(), OdsError> {
     if style.styleuse() == StyleUse::Default {
         xml_out.elem("style:default-style")?;
     } else {
@@ -1656,7 +1767,10 @@ fn write_rowstyle(style: &RowStyle, xml_out: &mut XmlOdsWriter) -> Result<(), Od
     Ok(())
 }
 
-fn write_colstyle(style: &ColStyle, xml_out: &mut XmlOdsWriter) -> Result<(), OdsError> {
+fn write_colstyle<W: Write + Seek>(
+    style: &ColStyle,
+    xml_out: &mut XmlOdsWriter<W>,
+) -> Result<(), OdsError> {
     if style.styleuse() == StyleUse::Default {
         xml_out.elem("style:default-style")?;
     } else {
@@ -1689,7 +1803,10 @@ fn write_colstyle(style: &ColStyle, xml_out: &mut XmlOdsWriter) -> Result<(), Od
     Ok(())
 }
 
-fn write_cellstyle(style: &CellStyle, xml_out: &mut XmlOdsWriter) -> Result<(), OdsError> {
+fn write_cellstyle<W: Write + Seek>(
+    style: &CellStyle,
+    xml_out: &mut XmlOdsWriter<W>,
+) -> Result<(), OdsError> {
     if style.styleuse() == StyleUse::Default {
         xml_out.elem("style:default-style")?;
     } else {
@@ -1742,9 +1859,9 @@ fn write_cellstyle(style: &CellStyle, xml_out: &mut XmlOdsWriter) -> Result<(),
     Ok(())
 }
 
-fn write_paragraphstyle(
+fn write_paragraphstyle<W: Write + Seek>(
     style: &ParagraphStyle,
-    xml_out: &mut XmlOdsWriter,
+    xml_out: &mut XmlOdsWriter<W>,
 ) -> Result<(), OdsError> {
     if style.styleuse() == StyleUse::Default {
         xml_out.elem("style:default-style")?;
@@ -1802,7 +1919,10 @@ fn write_paragraphstyle(
     Ok(())
 }
 
-fn write_textstyle(style: &TextStyle, xml_out: &mut XmlOdsWriter) -> Result<(), OdsError> {
+fn write_textstyle<W: Write + Seek>(
+    style: &TextStyle,
+    xml_out: &mut XmlOdsWriter<W>,
+) -> Result<(), OdsError> {
     if style.styleuse() == StyleUse::Default {
         xml_out.elem("style:default-style")?;
     } else {
@@ -1835,7 +1955,10 @@ fn write_textstyle(style: &TextStyle, xml_out: &mut XmlOdsWriter) -> Result<(),
     Ok(())
 }
 
-fn write_graphicstyle(style: &GraphicStyle, xml_out: &mut XmlOdsWriter) -> Result<(), OdsError> {
+fn write_graphicstyle<W: Write + Seek>(
+    style: &GraphicStyle,
+    xml_out: &mut XmlOdsWriter<W>,
+) -> Result<(), OdsError> {
     if style.styleuse() == StyleUse::Default {
         xml_out.elem("style:default-style")?;
     } else {
@@ -1869,11 +1992,11 @@ fn write_graphicstyle(style: &GraphicStyle, xml_out: &mut XmlOdsWriter) -> Resul
     Ok(())
 }
 
-fn write_valuestyles(
+fn write_valuestyles<W: Write + Seek>(
     value_formats: &HashMap<String, ValueFormat>,
     origin: StyleOrigin,
     styleuse: StyleUse,
-    xml_out: &mut XmlOdsWriter,
+    xml_out: &mut XmlOdsWriter<W>,
 ) -> Result<(), OdsError> {
     for value_format in value_formats
         .values()
@@ -1961,9 +2084,9 @@ fn write_valuestyles(
     Ok(())
 }
 
-fn write_pagestyles(
+fn write_pagestyles<W: Write + Seek>(
     styles: &HashMap<String, PageStyle>,
-    xml_out: &mut XmlOdsWriter,
+    xml_out: &mut XmlOdsWriter<W>,
 ) -> Result<(), OdsError> {
     for style in styles.values() {
         xml_out.elem("style:page-layout")?;
@@ -2000,9 +2123,9 @@ fn write_pagestyles(
     Ok(())
 }
 
-fn write_masterpage<'a>(
-    styles: &'a HashMap<String, MasterPage>,
-    xml_out: &mut XmlOdsWriter<'a>,
+fn write_masterpage<W: Write + Seek>(
+    styles: &HashMap<String, MasterPage>,
+    xml_out: &mut XmlOdsWriter<W>,
 ) -> Result<(), OdsError> {
     for style in styles.values() {
         xml_out.elem("style:master-page")?;
@@ -2061,7 +2184,10 @@ fn write_masterpage<'a>(
     Ok(())
 }
 
-fn write_regions<'a>(hf: &'a HeaderFooter, xml_out: &mut XmlOdsWriter<'a>) -> Result<(), OdsError> {
+fn write_regions<W: Write + Seek>(
+    hf: &HeaderFooter,
+    xml_out: &mut XmlOdsWriter<W>,
+) -> Result<(), OdsError> {
     if let Some(left) = hf.left() {
         xml_out.elem("style:region-left")?;
         write_xmltag(left, xml_out)?;
