@@ -143,10 +143,12 @@
 
 #![doc(html_root_url = "https://docs.rs/spreadsheet-ods/0.4.0")]
 
+use std::collections::btree_map::Range;
 use std::collections::{BTreeMap, HashMap};
 use std::convert::TryFrom;
 use std::fmt;
 use std::fmt::{Display, Formatter};
+use std::ops::RangeBounds;
 use std::str::FromStr;
 
 use chrono::Duration;
@@ -162,7 +164,7 @@ use crate::ds::detach::Detached;
 pub use crate::error::OdsError;
 pub use crate::format::{ValueFormat, ValueFormatRef};
 use crate::io::FileBuf;
-pub use crate::io::{read_ods, write_ods};
+pub use crate::io::{read_ods, read_ods_buf, write_ods, write_ods_buf};
 pub use crate::refs::{CellRange, CellRef, ColRange, RowRange};
 pub use crate::style::units::{Angle, Length};
 pub use crate::style::{CellStyle, CellStyleRef};
@@ -1334,6 +1336,14 @@ impl Sheet {
         self.data.get(&(row, col)).is_none()
     }
 
+    /// Allows access to SCells via ranges.
+    pub fn range<R>(&self, range: R) -> Range<'_, (ucell, ucell), SCell>
+    where
+        R: RangeBounds<(ucell, ucell)>,
+    {
+        self.data.range(range)
+    }
+
     /// Returns the cell if available.
     pub fn cell(&self, row: ucell, col: ucell) -> Option<&SCell> {
         self.data.get(&(row, col))
@@ -1755,7 +1765,7 @@ pub enum Value {
     Percentage(f64),
     Currency(String, f64),
     Text(String),
-    TextXml(Box<TextTag>),
+    TextXml(Vec<TextTag>),
     DateTime(NaiveDateTime),
     TimeDuration(Duration),
 }
@@ -1975,7 +1985,13 @@ impl From<&String> for Value {
 
 impl From<TextTag> for Value {
     fn from(t: TextTag) -> Self {
-        Value::TextXml(Box::new(t))
+        Value::TextXml(vec![t])
+    }
+}
+
+impl From<Vec<TextTag>> for Value {
+    fn from(t: Vec<TextTag>) -> Self {
+        Value::TextXml(t)
     }
 }
 
